@@ -31,11 +31,11 @@
 
 #include "kernel-lib/list.h"
 #include "kernel-lib/radix-tree.h"
-#include "ctree.h"
-#include "extent-cache.h"
-#include "disk-io.h"
-#include "volumes.h"
-#include "transaction.h"
+#include "kernel-shared/ctree.h"
+#include "common/extent-cache.h"
+#include "kernel-shared/disk-io.h"
+#include "kernel-shared/volumes.h"
+#include "kernel-shared/transaction.h"
 #include "crypto/crc32c.h"
 #include "common/utils.h"
 #include "check/common.h"
